@@ -20,17 +20,20 @@ class Compiler(AsmFile):
     assembled_data = None
     bad_chars = None
     remove_bad_chars = False
+    writable_text = False
 
     def __init__(self, filename: str,
                  assembled_data: bytearray,
                  bad_chars: [bytearray, bytes] = bytearray(),
-                 remove_bad_chars: bool = False):
+                 remove_bad_chars: bool = False,
+                 writable_text: bool = False):
         super().__init__(filename)
         self.assembled_data = assembled_data
         self.c_file = Path(f"{self.file_pattern}.c")
         self.bin_file = Path(f"{self.get_name()}")
         self.bad_chars = bytearray([b for b in bad_chars])
         self.remove_bad_chars = remove_bad_chars
+        self.writable_text = writable_text
 
     def get_name(self) -> str:
         import platform
@@ -55,12 +58,44 @@ class Compiler(AsmFile):
 
             with open(self.c_file, 'w', encoding="utf-8") as f:
                 f.write('#include<stdio.h>\n')
+                f.write('#include<stdlib.h>\n')
+                f.write('#include<unistd.h>\n')
+                f.write('#include<errno.h>\n')
                 f.write('#include<string.h>\n')
+                f.write('#include<time.h>\n')
+                f.write('\n')
+                f.write('#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)\n')
+                f.write('    #include <windows.h>\n')
+                f.write('#elif __APPLE__\n')
+                f.write('    #include <TargetConditionals.h>\n')
+                f.write('    #if TARGET_IPHONE_SIMULATOR\n')
+                f.write('         // iOS, tvOS, or watchOS Simulator\n')
+                f.write('    #elif TARGET_OS_MACCATALYST\n')
+                f.write('         // Macs Catalyst (ports iOS API into Mac, like UIKit).\n')
+                f.write('    #elif TARGET_OS_IPHONE\n')
+                f.write('        // iOS, tvOS, or watchOS device\n')
+                f.write('    #elif TARGET_OS_MAC\n')
+                f.write('        // Other kinds of Apple platforms\n')
+                f.write('    #else\n')
+                f.write('    #   error "Unknown Apple platform"\n')
+                f.write('    #endif\n')
+                f.write('#elif __linux__\n')
+                f.write('    // linux\n')
+                f.write('    #include <sys/mman.h>\n')
+                f.write('#elif __unix__ // all unices not caught above\n')
+                f.write('    // Unix\n')
+                f.write('#elif defined(_POSIX_VERSION)\n')
+                f.write('    // POSIX\n')
+                f.write('#else\n')
+                f.write('#   error "Unknown compiler"\n')
+                f.write('#endif\n')
                 f.write('\n')
                 f.write('char ZEROARRAY[1024] = {0};\n')
                 f.write('\n')
                 f.write('void shell(int*, int*);\n')
                 f.write('void code_cave();\n')
+                f.write('int wtext();\n')
+                f.write('int end_of_code();\n')
                 f.write('\n')
                 f.write('int main() {\n')
                 f.write('\n')
@@ -76,10 +111,38 @@ class Compiler(AsmFile):
                     f.write('    printf("rsi = %p => writable memory (%zd bytes)\\n", &ZEROARRAY, size2);\n')
 
                 f.write('\n')
+                f.write('    wtext();\n')
                 f.write('    shell(&code_cave, &ZEROARRAY);\n')
                 f.write('    return 0;\n')
                 f.write('}\n')
                 f.write('\n')
+
+                f.write('int wtext(){\n')
+                if self.writable_text:
+                    f.write('    void       *const target = &shell;\n')
+                    f.write('    size_t      length = (size_t)((intptr_t) end_of_code - (intptr_t)shell);\n')
+                    f.write('    #if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)\n')
+                    f.write('        SYSTEM_INFO si;\n')
+                    f.write('        GetSystemInfo(&si);\n')
+                    f.write('        const long  page = si.dwPageSize;\n')
+                    f.write('        DWORD l=0;\n')
+                    f.write('        void       *start = (char *)target - ((long)target % page);\n')
+                    f.write('        size_t      bytes = length + (size_t)((long)target % page);\n')
+                    f.write('        VirtualProtect(start, bytes, PAGE_EXECUTE_READWRITE, &l);\n')
+                    f.write('    #elif __linux__\n')
+                    f.write('        const long  page = sysconf(_SC_PAGESIZE);\n')
+                    f.write('        void       *start = (char *)target - ((long)target % page);\n')
+                    f.write('        size_t      bytes = length + (size_t)((long)target % page);\n')
+                    f.write('        if (mprotect(start, page, PROT_READ | PROT_WRITE | PROT_EXEC))\n')
+                    f.write('            return errno;\n')
+                    f.write('    #else\n')
+                    f.write('        return errno;\n')
+                    f.write('    #endif\n')
+                else:
+                    f.write('        return 0;\n')
+                f.write('}\n')
+                f.write('\n')
+
                 f.write('void shell(int *code_cave, int *data) {\n')
 
                 if self.arch == 'x86':
@@ -117,6 +180,13 @@ class Compiler(AsmFile):
                     f.write('    asm("NOP");\n')
 
                 f.write('}\n')
+
+                f.write('int end_of_code(){\n')
+                f.write('   int i = 0;\n')
+                f.write('   ++i + time(0); // time(0) is to prevent optimizer from just doing: return 1;\n')
+                f.write('   return i;\n')
+                f.write('}\n')
+
                 pass
 
         except IOError as x:
